@@ -8,19 +8,23 @@ use 5.010000;
 use Exporter qw( import );
 our @EXPORT_OK = qw( choose_videos set_sort_videolist );
 
+use Encode qw( decode );
+
+use Encode::Locale         qw();
 use List::MoreUtils        qw( any none );
 use Term::Choose           qw( choose );
 use Term::ReadLine::Simple qw();
 
+use App::YTDL::Helper qw( encode_list_for_lax_stdout );
+
 
 
 sub _my_sort {
-    my ( $opt, $h_ref, $a, $b ) = @_;
-    my $item = $opt->{list_sort_item};
-    my @s = $opt->{list_sort_order} eq 'Asc'
-                ? ( $h_ref->{$a}{$item}, $h_ref->{$b}{$item} )
-                : ( $h_ref->{$b}{$item}, $h_ref->{$a}{$item} );
-    if ( $item eq 'view_count_raw' ) {
+    my ( $sort_item, $sort_order, $h_ref, $a, $b ) = @_;
+    my @s = $sort_order eq 'Asc'
+                ? ( $h_ref->{$a}{$sort_item}, $h_ref->{$b}{$sort_item} )
+                : ( $h_ref->{$b}{$sort_item}, $h_ref->{$a}{$sort_item} );
+    if ( $sort_item eq 'view_count_raw' ) {
        $s[0] <=> $s[1] || $h_ref->{$a}{title} cmp $h_ref->{$b}{title};
     }
     else {
@@ -31,11 +35,27 @@ sub _my_sort {
 
 sub choose_videos {
     my ( $opt, $info, $tmp, $ex, $prompt ) = @_;
+    my $length_view_count = 0;
+    my $length_video_id   = 0;
+    my $has_upload_date   = 0;
+    my $has_view_count    = 0;
+    my $has_duration      = 0;
+    for my $video_id ( keys %{$tmp->{$ex}} ) {
+        $has_view_count++  if $tmp->{$ex}{$video_id}{view_count};
+        $has_upload_date++ if $tmp->{$ex}{$video_id}{upload_date} ne '0000-00-00';
+        $has_duration++    if $tmp->{$ex}{$video_id}{duration}    ne '-:--:--';
+        if ( length $video_id > $length_video_id ) {
+            $length_video_id = length $video_id;
+        }
+        if ( length $tmp->{$ex}{$video_id}{view_count} > $length_view_count ) {
+            $length_view_count = length $tmp->{$ex}{$video_id}{view_count};
+        }
+    }
     my $regexp;
-    my $back   = '       BACK';
-    my $filter = '     FILTER';
-    my $sort   = '       SORT';
-    my $enter  = '      ENTER';
+    my $back   = sprintf "%*s", $length_video_id, '   BACK';
+    my $filter = sprintf "%*s", $length_video_id, ' FILTER';
+    my $sort   = sprintf "%*s", $length_video_id, '   SORT';
+    my $enter  = sprintf "%*s", $length_video_id, '  ENTER';
     my %chosen_video_ids;
     my @last_chosen_video_ids = ();
 
@@ -45,7 +65,19 @@ sub choose_videos {
         my @tmp_video_ids;
         my $index = $#pre;
         my $mark = [];
-        my @video_ids = sort { _my_sort( $opt, $tmp->{$ex}, $a, $b ) }  keys %{$tmp->{$ex}};
+        my $sort_item  = $opt->{list_sort_item};
+        my $sort_order = $opt->{list_sort_order};
+        if ( $sort_item eq 'view_count_raw' && ! $has_view_count ) {
+            $sort_item = 'upload_date';
+        }
+        if ( $sort_item eq 'duration' && ! $has_duration ) {
+            $sort_item = 'upload_date';
+        }
+        if ( $sort_item eq 'upload_date' && ! $has_upload_date ) {
+            $sort_item = 'title';
+            $sort_order = 'Asc';
+        }
+        my @video_ids = sort { _my_sort( $sort_item, $sort_order, $tmp->{$ex}, $a, $b ) }  keys %{$tmp->{$ex}};
 
         VIDEO_ID: for my $video_id ( @video_ids ) {
             my $title = $tmp->{$ex}{$video_id}{title};
@@ -62,13 +94,17 @@ sub choose_videos {
             if ( $ex eq 'youtube' ) {
                 $tmp->{$ex}{$video_id}{extractor} = 'youtube';
             }
-            if ( $opt->{list_sort_item} eq 'view_count_raw' || $opt->{show_view_count} ) {
-                push @videos, sprintf "%11s | %7s  %10s  %$opt->{length_view_count}s  %s", $video_id,
-                    $tmp->{$ex}{$video_id}{duration}, $tmp->{$ex}{$video_id}{upload_date}, $tmp->{$ex}{$video_id}{view_count}, $title;
-            }
-            else {
-                push @videos, sprintf "%11s | %7s  %10s  %s", $video_id, $tmp->{$ex}{$video_id}{duration}, $tmp->{$ex}{$video_id}{upload_date}, $title;
+            if ( $Encode::Locale::ENCODING_CONSOLE_OUT !~ /^UTF-/i ) {
+                ( $title ) = encode_list_for_lax_stdout( $opt, $title );
+                $title = decode( 'console_out', $title );
             }
+            $has_view_count = 0 if $opt->{list_sort_item} ne 'view_count_raw' && ! $opt->{show_view_count};
+            my $line = sprintf "%*s |", $length_video_id, $video_id;
+            $line .= sprintf " %7s", $tmp->{$ex}{$video_id}{duration}                        if $has_duration;
+            $line .= sprintf "  %10s", $tmp->{$ex}{$video_id}{upload_date}                   if $has_upload_date;
+            $line .= sprintf "  %*s", $length_view_count, $tmp->{$ex}{$video_id}{view_count} if $has_view_count;
+            $line .= sprintf "  %s", $title;
+            push @videos, $line;
             push @tmp_video_ids, $video_id;
             $index++;
             push @$mark, $index if any { $video_id eq $_ } keys %chosen_video_ids;
@@ -150,10 +186,10 @@ sub set_sort_videolist {
     my $sort_items = [ 'upload_date', 'title', 'view_count_raw', 'duration' ];
     my $confirm = '  CONFIRM';
     my @pre = ( undef, $confirm );
-    my $choices = [ @pre, map { my $s = ucfirst; $s =~ s/_raw\z//; $s =~ s/_/ /g; '- ' . $s } @$sort_items ];
+    my $choices = [ @pre, map { my $s = $_; $s =~ s/_raw\z//; $s =~ s/_/ /g; '- ' . $s } @$sort_items ];
 
     ITEM: while ( 1 ) {
-        my $current = ucfirst $opt->{list_sort_item};
+        my $current = $opt->{list_sort_item};
         $current =~ s/_raw\z//;
         $current =~ s/_/ /g;
         my $prompt = sprintf 'Sort item: [%s]', $current;
@@ -173,7 +209,7 @@ sub set_sort_videolist {
     }
 
     ORDER: while ( 1 ) {
-        my $curr_sort_item = ucfirst $opt->{list_sort_item};
+        my $curr_sort_item = $opt->{list_sort_item};
         $curr_sort_item =~ s/_raw\z//;
         $curr_sort_item =~ s/_/ /g;
         my $order_prompt = sprintf 'Sort order "%s": [%s]', $curr_sort_item, $opt->{list_sort_order};
