@@ -22,9 +22,49 @@ sub read_history_files {
     if ( $opt->{max_channels} ) {
         if ( -e $opt->{s_channel_file} ) {
             $opt->{channel_sticky} = read_json( $opt->{s_channel_file} );
+
+            ### keep this some time ###############################################################
+            for my $url ( keys %{$opt->{channel_sticky}} ) {
+                $opt->{channel_sticky}{$url}{uploader}    =         delete $opt->{channel_sticky}{$url}{channel}       if $opt->{channel_sticky}{$url}{channel};
+                $opt->{channel_sticky}{$url}{uploader_id} =         delete $opt->{channel_sticky}{$url}{channel_id}    if $opt->{channel_sticky}{$url}{channel_id};
+                $opt->{channel_sticky}{$url}{extractor}   = lcfirst delete $opt->{channel_sticky}{$url}{extractor_key} if $opt->{channel_sticky}{$url}{extractor_key};
+                delete $opt->{channel_sticky}{$url}{channel_url};
+                if ( $opt->{channel_sticky}{$url}{extractor} eq 'youtube' ) {
+                    if ( $url =~ /^c#([^#]+)\z/ ) {
+                        my $new_url = "https://www.youtube.com/user/$1";
+                        $opt->{channel_sticky}{$new_url} = delete $opt->{channel_sticky}{$url};
+                    }
+                }
+                if ( $url =~ /^http:(.+)\z/ ) {
+                    my $new_url = "https:$1";
+                    $opt->{channel_sticky}{$new_url} = delete $opt->{channel_sticky}{$url};
+                }
+            }
+            #######################################################################################
+
         }
         if ( -e $opt->{h_channel_file} ) {
             $opt->{channel_history} = read_json( $opt->{h_channel_file} );
+
+            ### keep this some time ###############################################################
+            for my $url ( keys %{$opt->{channel_history}} ) {
+                $opt->{channel_history}{$url}{uploader}    =         delete $opt->{channel_history}{$url}{channel}       if $opt->{channel_history}{$url}{channel};
+                $opt->{channel_history}{$url}{uploader_id} =         delete $opt->{channel_history}{$url}{channel_id}    if $opt->{channel_history}{$url}{channel_id};
+                $opt->{channel_history}{$url}{extractor}   = lcfirst delete $opt->{channel_history}{$url}{extractor_key} if $opt->{channel_history}{$url}{extractor_key};
+                delete $opt->{channel_history}{$url}{channel_url};
+                if ( $opt->{channel_history}{$url}{extractor} eq 'youtube' ) {
+                    if ( $url =~ /^c#([^#]+)\z/ ) {
+                        my $new_url = "https://www.youtube.com/user/$1";
+                        $opt->{channel_history}{$new_url} = delete $opt->{channel_history}{$url};
+                    }
+                }
+                if ( $url =~ /^http:(.+)\z/ ) {
+                    my $new_url = "https:$1";
+                    $opt->{channel_history}{$new_url} = delete $opt->{channel_history}{$url};
+                }
+            }
+            #######################################################################################
+
         }
     }
 }
@@ -78,15 +118,15 @@ sub channel_history_menu {
                         $i -= @pre;
                         if ( $i <= $#$sticky ) {
                             push @ids, $sticky_url->[$i];
-                            $prompt .= sprintf "= %s (%s)\n", $sticky->[$i], $opt->{channel_sticky}{$sticky_url->[$i]}{channel_id};
+                            $prompt .= sprintf "= %s (%s)\n", $sticky->[$i], $opt->{channel_sticky}{$sticky_url->[$i]}{uploader_id};
                         }
                         else {
                             $i -= @$sticky;
                             push @ids, $history_url->[$i];
-                            $prompt .= sprintf "= %s (%s)\n", $history->[$i], $opt->{channel_history}{$history_url->[$i]}{channel_id};
+                            $prompt .= sprintf "= %s (%s)\n", $history->[$i], $opt->{channel_history}{$history_url->[$i]}{uploader_id};
                         }
                     }
-                    $opt->{nr_of_latest_videos} = $list_size[$list_size_idx] =~ /\D(\d+)/ ? $1 : 0;
+                    $opt->{max_videos} = $list_size[$list_size_idx] =~ /\D(\d+)/ ? $1 : 0;
                     return @ids;
                 }
                 else {
@@ -94,12 +134,12 @@ sub channel_history_menu {
                         $i -= @pre;
                         if ( $i <= $#$sticky ) {
                             push @ids, $sticky_url->[$i];
-                            $prompt .= sprintf "= %s (%s)\n", $sticky->[$i], $opt->{channel_sticky}{$sticky_url->[$i]}{channel_id};
+                            $prompt .= sprintf "= %s (%s)\n", $sticky->[$i], $opt->{channel_sticky}{$sticky_url->[$i]}{uploader_id};
                         }
                         else {
                             $i -= @$sticky;
                             push @ids, $history_url->[$i];
-                            $prompt .= sprintf "= %s (%s)\n", $history->[$i], $opt->{channel_history}{$history_url->[$i]}{channel_id};
+                            $prompt .= sprintf "= %s (%s)\n", $history->[$i], $opt->{channel_history}{$history_url->[$i]}{uploader_id};
                         }
                     }
                 }
@@ -109,6 +149,7 @@ sub channel_history_menu {
             my ( $sticky,  $sticky_url )  = _sticky_name_and_url( $opt->{channel_sticky} );
             my ( $history, $history_url ) = _history_name_and_url( $opt, $opt->{channel_history} );
             my $changed = 0;
+
             STICKY: while ( 1 ) {
                 my @pre = ( undef );
                 my $idx = choose(
@@ -153,8 +194,8 @@ sub _channel_choices {
     my ( $pre, $sticky, $history, $removed_next ) = @_;
     my $choices = [ @$pre, map( "* $_", @$sticky ) ];
     for my $i ( 0 .. $#$history ) {
-            push @$choices, "  $history->[$i]" if ! $removed_next->[$i];
-            push @$choices, "| $history->[$i]" if   $removed_next->[$i];
+        push @$choices, "  $history->[$i]" if ! $removed_next->[$i];
+        push @$choices, "| $history->[$i]" if   $removed_next->[$i];
     }
     return $choices;
 }
@@ -168,7 +209,7 @@ sub _history_name_and_url {
     my $count = 0;
     for my $key ( sort { $ref->{$b}{timestamp} <=> $ref->{$a}{timestamp} } keys %$ref ) {
         $count++;
-        push @$tmp_name, $ref->{$key}{channel};
+        push @$tmp_name, $ref->{$key}{uploader};
         push @$tmp_url, $key;
         push @$tmp_removed_next, $count >= $opt->{max_channels} ? 1 : 0;
     }
@@ -195,8 +236,8 @@ sub _sticky_name_and_url {
     my ( $ref ) = @_;
     my $sticky_name = [];
     my $sticky_url  = [];
-    for my $key ( sort { lc $ref->{$a}{channel} cmp lc $ref->{$b}{channel} } keys %$ref ) {
-        push @$sticky_name, $ref->{$key}{channel};
+    for my $key ( sort { lc $ref->{$a}{uploader} cmp lc $ref->{$b}{uploader} } keys %$ref ) {
+        push @$sticky_name, $ref->{$key}{uploader};
         push @$sticky_url, $key;
     }
     return $sticky_name, $sticky_url;
@@ -205,22 +246,20 @@ sub _sticky_name_and_url {
 
 sub add_channel_to_history {
     my ( $opt, $info, $ex, $video_id ) = @_;
-    my $channel    = $info->{$ex}{$video_id}{uploader};
-    my $channel_id = $info->{$ex}{$video_id}{uploader_id};
-    if ( ! length $channel_id ) {
-        return;
-    }
-    my $channel_url;
-    $channel_url = 'c#'                if $ex eq 'youtube';
-    $channel_url = 'http://vimeo.com/' if $ex eq 'vimeo'; #
-    if ( defined $channel_url ) {
-        $channel_url .= $channel_id;
-        if ( none{ $channel_url eq $_ } keys %{$opt->{channel_sticky}} ) {
-            $opt->{channel_history}{$channel_url} = {
-                channel     => $channel,
-                channel_id  => $channel_id,
-                extractor   => $ex,
-                timestamp   => time(),
+    my $uploader    = $info->{$ex}{$video_id}{uploader};
+    my $uploader_id = $info->{$ex}{$video_id}{uploader_id};
+    return if ! length $uploader_id;
+    my $uploader_url;
+    $uploader_url = 'https://www.youtube.com/user/' if $ex eq 'youtube';
+    $uploader_url = 'https://vimeo.com/'            if $ex eq 'vimeo';
+    if ( defined $uploader_url ) {
+        $uploader_url .= $uploader_id;
+        if ( none{ $uploader_url eq $_ } keys %{$opt->{channel_sticky}} ) {
+            $opt->{channel_history}{$uploader_url} = {
+                uploader     => $uploader,
+                uploader_id  => $uploader_id,
+                extractor    => $ex,
+                timestamp    => time(),
             };
         }
     }
