@@ -11,19 +11,18 @@ our @EXPORT_OK = qw( read_config_file set_options );
 use File::Spec::Functions qw( catfile );
 use File::Temp            qw();
 use FindBin               qw( $RealBin $RealScript );
-use List::Util            qw( max );
 use Pod::Usage            qw( pod2usage );
 
-use File::Which            qw( which );
-use IPC::System::Simple    qw( capture );
 use Term::Choose           qw( choose );
-use Term::Choose::Util     qw( print_hash choose_a_dir choose_a_number insert_sep );
+use Term::Choose::Util     qw( print_hash choose_a_dir choose_a_number choose_multi insert_sep );
 use Term::ReadLine::Simple qw();
 use Text::LineFold         qw();
 use Try::Tiny              qw( try catch );
 
-use App::YTDL::Helper        qw( encode_fs write_json read_json HIDE_CURSOR SHOW_CURSOR );
-use App::YTDL::Choose_Videos qw( set_sort_videolist );
+use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
+
+use App::YTDL::ChooseVideos qw( set_sort_videolist );
+use App::YTDL::Helper       qw( write_json read_json uni_capture HIDE_CURSOR SHOW_CURSOR );
 
 
 
@@ -31,15 +30,15 @@ sub _show_info {
     my ( $opt ) = @_;
     my ( $youtube_dl_version, $ffmpeg_version, $ffprobe_version );
     try {
-        my $youtube_dl = which( 'youtube-dl' ) // 'youtube-dl';
-        $youtube_dl_version = capture( $youtube_dl, '--version' );
+        my @cmd = ( $opt->{youtube_dl}, '--version' );
+        $youtube_dl_version = uni_capture( @cmd );
     }
     catch {
         $youtube_dl_version = $_;
     };
     try {
-        my $ffmpeg = which( 'ffmpeg' );
-        $ffmpeg_version = capture( $ffmpeg, '-version' );
+        my @cmd = ( $opt->{ffmpeg}, '-version' );
+        $ffmpeg_version = uni_capture( @cmd );
         if ( $ffmpeg_version =~ /^ffmpeg version (\S+)/m ) {
             $ffmpeg_version = $1;
         }
@@ -48,8 +47,8 @@ sub _show_info {
         }
     };
     try {
-        my $ffprobe = which( 'ffprobe' );
-        $ffprobe_version = capture( $ffprobe, '-version' );
+        my @cmd = ( $opt->{ffprobe}, '-version' );
+        $ffprobe_version = uni_capture( @cmd );
         if ( $ffprobe_version =~ /^ffprobe version (\S+)/m ) {
             $ffprobe_version = $1;
         }
@@ -83,40 +82,17 @@ sub _show_info {
 sub _menus {
     my $menus = {
         main => [
-            [ 'show_help_text',  "  HELP"       ],
-            [ 'show_info',       "  INFO"       ],
-            [ 'group_directory', "- Directory"  ],
-            [ 'group_file',      "- File"       ],
-            [ 'group_quality',   "- Quality"    ],
-            [ 'group_download',  "- Download"   ],
-            [ 'group_merge',     "- Merge"      ],
-            [ 'group_history',   "- History"    ],
-            [ 'group_list_menu', "- Video List" ],
-            [ 'group_output',    "- Appearance" ],
-            [ 'list_extractors', "  Extractors" ],
-        ],
-        group_download => [
-            [ 'useragent',      "- UserAgent"        ],
-            [ 'overwrite',      "- Overwrite"        ],
-            [ 'retries',        "- Download retries" ],
-            [ 'timeout',        "- Timeout"          ],
-        ],
-        group_merge => [
-            [ 'merge_enabled',    "- Enable Merge"      ],
-            [ 'merge_overwrite',  "- ffmpeg overwrites" ],
-            [ 'merge_ext',        "- Output formats"    ],
-            [ 'merge_loglevel',   "- Verbosity"         ],
-            [ 'merged_in_files',  "- Input files"       ],
-        ],
-        group_quality => [
-            [ 'auto_quality',    "- Auto quality mode"            ],
-            [ 'print_pref_qual', "- Preferred qualities"          ],
-            [ 'pref_qual_slots', "- Slots 'preferred qualilties'" ],
-        ],
-        group_history => [
-            [ 'log_info',             "- Logging"         ],
-            [ 'max_channels',         "- Channel history" ],
-            [ 'channel_by_timestamp', "- History sort"    ],
+            [ 'show_help_text',   "  HELP"       ],
+            [ 'show_info',        "  INFO"       ],
+            [ 'group_directory',  "- Directory"  ],
+            [ 'group_file',       "- File"       ],
+            [ 'group_quality',    "- Quality"    ],
+            [ 'group_download',   "- Download"   ],
+            [ 'group_merge',      "- Merge"      ],
+            [ 'group_history',    "- History"    ],
+            [ 'group_list_menu',  "- Video List" ],
+            [ 'group_output',     "- Output"     ],
+            [ 'group_youtube_dl', "- Youtube-dl" ],
         ],
         group_directory => [
             [ 'video_dir',     "- Main video directory" ],
@@ -124,23 +100,66 @@ sub _menus {
             [ 'channel_dir',   "- Channel directory"    ],
         ],
         group_file => [
-            [ 'max_len_f_name',   "- Max filename length" ],
-            [ 'modify_timestamp', "- File timestamp"      ],
+            [ 'max_len_f_name',    "- Max filename length" ],
+            [ 'replace_spaces',    "- Replace spaces"      ],
+            [ 'sanitize_filename', "- Sanitize filename"   ],
+            [ 'modify_timestamp',  "- File timestamp"      ],
         ],
+        group_quality => [
+            [ 'auto_quality',    "- Auto quality mode"            ],
+            [ 'pref_qual_slots', "- Slots 'preferred qualilties'" ],
+            [ 'print_pref_qual', "  Preferred qualities"          ],
+        ],
+        group_download => [
+            [ 'useragent', "- UserAgent"        ],
+            [ 'overwrite', "- Overwrite"        ],
+            [ 'retries',   "- Download retries" ],
+            [ 'timeout',   "- Timeout"          ],
+        ],
+        group_merge => [
+            [ 'merge_enabled',   "- Enable Merge"      ],
+            [ 'merge_overwrite', "- ffmpeg overwrites" ],
+            [ 'merge_ext',       "- Output formats"    ],
+            [ 'merge_loglevel',  "- Verbosity"         ],
+            [ 'merged_in_files', "- Input files"       ],
+        ],
+        group_history => [
+            [ 'log_info',             "- Logging"         ],
+            [ 'max_channels',         "- Channel history" ],
+            [ 'channel_by_timestamp', "- History sort"    ],
+        ],
+
         group_list_menu => [
-            [ 'list_size_idx',   "- List size"  ],
-            [ 'list_sort_item',  "- Sort order" ],
-            [ 'show_view_count', "- View count" ],
+            [ 'list_size_idx',   "- List size"       ],
+            [ 'list_sort_item',  "- Sort order"      ],
+            [ 'show_view_count', "- Show view count" ],
+            [ '_fast_listmenu',  "- Fast list-menu"  ],
         ],
         group_output => [
-            [ 'max_info_width',  "- Max info width" ],
-            [ 'kb_sec_len',      "- Digits 'k/s'"   ],
+            [ 'text_unidecode', "- Unmappable characters" ],
+            [ 'max_info_width', "- Max info width"        ],
+            [ 'kb_sec_len',     "- Digits 'k/s'"          ],
+        ],
+        group_youtube_dl => [
+            [ 'use_netrc',        "- Use '.netrc' data" ],
+            [ 'avail_extractors', "  Extractors"        ],
         ],
     };
     return $menus;
 }
 
 
+sub _sub_menus {
+    my ( $key ) = @_;
+    my $sub_menus = {
+        _fast_listmenu => [ [ 'fast_list_youtube', "- youtube", [ 'NO', 'YES' ] ],
+                            [ 'fast_list_vimeo',   "- vimeo",   [ 'NO', 'YES' ] ], ],
+    };
+    return $sub_menus->{$key} if $key;
+    return $sub_menus;
+}
+
+
 sub _group_prompt {
     my ( $group ) = @_;
     my $group_prompt ={
@@ -159,11 +178,15 @@ sub _group_prompt {
 
 sub set_options {
     my ( $opt ) = @_;
-    my $menus = _menus();
+    my $menus     = _menus();
+    my $sub_menus = _sub_menus();
     my @keys;
     for my $group ( keys %$menus ) {
         next if $group eq 'main';
-        push @keys, map { $_->[0] } @{$menus->{$group}};
+        push @keys, grep { ! /^_/ } map { $_->[0] } @{$menus->{$group}};
+    }
+    for my $key ( %$sub_menus ) {
+        push @keys, map { $_->[0] } @{$sub_menus->{$key}};
     }
     my $group = 'main';
     my $backup_old_idx = 0;
@@ -220,14 +243,13 @@ sub set_options {
             elsif ( $choice eq "show_info" ) {
                 _show_info( $opt );
             }
-            elsif ( $choice eq "list_extractors" ) {
+            elsif ( $choice eq "avail_extractors" ) {
                 print HIDE_CURSOR;
                 say 'Close with ENTER';
                 my @capture;
                 try {
-                    my $youtube_dl = which( 'youtube-dl' ) // 'youtube-dl';
-                    @capture = capture( $youtube_dl, '--list-extractors' );
-
+                    my @cmd = ( $opt->{youtube_dl}, '--list-extractors' );
+                    @capture = uni_capture( @cmd );
                 }
                 catch {
                     say "List extractors: $_";
@@ -240,71 +262,52 @@ sub set_options {
                     );
                 }
             }
-            elsif ( $choice eq "useragent" ) {
-                my $prompt = 'Set the UserAgent';
-                my $list = [
-                    [ 'UserAgent', $opt->{useragent} ]
-                ];
-                _local_readline( $opt, $choice, $prompt, $list );
-                $opt->{useragent} = 'Mozilla/5.0' if $opt->{useragent} eq '';
-                $opt->{useragent} = ''            if $opt->{useragent} eq '""';
+            elsif ( $choice eq "video_dir" ) {
+                my $prompt = 'Video directory';
+                _opt_choose_a_directory( $opt, $choice, $prompt );
             }
-            elsif ( $choice eq "overwrite" ) {
-                my $prompt = 'Overwrite files';
+            elsif ( $choice eq "extractor_dir" ) {
+                my $prompt = 'Use extractor directories';
+                my $list = [ qw( no yes ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
+            elsif ( $choice eq "channel_dir" ) {
+                my $prompt = 'Use channel directories';
                 my $list = [
-                    'NO (append)',
-                    'YES'
+                    'no',
+                    'if from channel or playlist',
+                    'always',
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "retries" ) {
-                my $prompt = 'Download retries';
+            elsif ( $choice eq "max_len_f_name" ) {
+                my $prompt = 'Max filename length';
                 my $digits = 3;
                 _opt_number_range( $opt, $choice, $prompt, $digits );
             }
-            elsif ( $choice eq "timeout" ) {
-                my $prompt = 'Connection timeout (s)';
-                my $digits = 3;
-                _opt_number_range( $opt, $choice, $prompt, $digits );
+            elsif ( $choice eq "replace_spaces" ) {
+                my $prompt = 'Replace spaces in filenames';
+                my $list = [ qw( no yes ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "merge_enabled" ) {
-                my $prompt = 'Enable merge';
+            elsif ( $choice eq "sanitize_filename" ) {
+                my $prompt = 'Sanitize filenames';
                 my $list = [
-                    'NO',
-                    'Ask',
-                    'YES'
+                    'replace  /',
+                    'replace  / \ : " * ? < > |',
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "merge_overwrite" ) {
-                my $prompt = '"ffmpeg": overwrite files without asking?';
-                my $list = [ qw( NO YES ) ];
-                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
-            }
-            elsif ( $choice eq "merge_ext" ) {
-                my $prompt = 'Output format of merged files';
-                my $list = [ qw( mkv mp4 ogg webm flv ) ];
-                _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
-            }
-            elsif ( $choice eq "merge_loglevel" ) {
-                my $prompt = '\'ffmpeg\' verbosity';
-                my $list = [ qw( error warning info ) ];
-                _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
-            }
-            elsif ( $choice eq "merged_in_files" ) {
-                my $prompt = 'The merged input files:';
-                my $list = [
-                    'Keep',
-                    'Move to "' . $opt->{dir_stream_files} . '"',
-                    'Remove'
-                ];
+            elsif ( $choice eq "modify_timestamp" ) {
+                my $prompt = 'Timestamp to upload date';
+                my $list = [ qw( no yes ) ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
             elsif ( $choice eq "auto_quality" ) {
                 my $prompt = 'Auto quality';
                 my $list = [
                     'manually',
-                    'keep_list_channel',
+                    'keep_channel_playlist',
                     'keep_extractor',
                     'preferred',
                     'default',
@@ -332,9 +335,65 @@ sub set_options {
                 my $digits = 2;
                 _opt_number_range( $opt, $choice, $prompt, $digits );
             }
+            elsif ( $choice eq "useragent" ) {
+                my $prompt = 'Set the UserAgent';
+                my $list = [
+                    [ 'UserAgent', $opt->{useragent} ]
+                ];
+                _local_readline( $opt, $choice, $prompt, $list );
+                $opt->{useragent} = 'Mozilla/5.0' if $opt->{useragent} eq '';
+                $opt->{useragent} = ''            if $opt->{useragent} eq '""';
+            }
+            elsif ( $choice eq "overwrite" ) {
+                my $prompt = 'Overwrite files';
+                my $list = [
+                    'no (append)',
+                    'yes'
+                ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
+            elsif ( $choice eq "retries" ) {
+                my $prompt = 'Download retries';
+                my $digits = 3;
+                _opt_number_range( $opt, $choice, $prompt, $digits );
+            }
+            elsif ( $choice eq "timeout" ) {
+                my $prompt = 'Connection timeout (s)';
+                my $digits = 3;
+                _opt_number_range( $opt, $choice, $prompt, $digits );
+            }
+            elsif ( $choice eq "merge_enabled" ) {
+                my $prompt = 'Enable merge';
+                my $list = [ qw( no ask yes ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
+            elsif ( $choice eq "merge_overwrite" ) {
+                my $prompt = '\'ffmpeg\' - overwrite files';
+                my $list = [ qw( ask yes ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
+            elsif ( $choice eq "merge_ext" ) {
+                my $prompt = 'Output format of merged files';
+                my $list = [ qw( mkv mp4 ogg webm flv ) ];
+                _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
+            }
+            elsif ( $choice eq "merge_loglevel" ) {
+                my $prompt = '\'ffmpeg\' verbosity';
+                my $list = [ qw( error warning info ) ];
+                _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
+            }
+            elsif ( $choice eq "merged_in_files" ) {
+                my $prompt = 'The merged input files:';
+                my $list = [
+                    'keep',
+                    'move to "' . $opt->{dir_stream_files} . '"',
+                    'remove'
+                ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
             elsif ( $choice eq "log_info" ) {
                 my $prompt = 'Enable info-logging';
-                my $list = [ qw( NO YES ) ];
+                my $list = [ qw( no yes ) ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
             elsif ( $choice eq "max_channels" ) {
@@ -345,44 +404,16 @@ sub set_options {
             elsif ( $choice eq "channel_by_timestamp" ) {
                 my $prompt = 'Sort channel history';
                 my $list = [
-                    'By name',
-                    'By timestamp'
+                    'by name',
+                    'by timestamp'
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "video_dir" ) {
-                my $prompt = 'Video directory';
-                _opt_choose_a_directory( $opt, $choice, $prompt );
-            }
-            elsif ( $choice eq "extractor_dir" ) {
-                my $prompt = 'Use extractor directories';
-                my $list = [ qw( NO YES ) ];
-                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
-            }
-            elsif ( $choice eq "channel_dir" ) {
-                my $prompt = 'Use channel directories';
-                my $list = [
-                    'No',
-                    'If from channel or list',
-                    'Always',
-                ];
-                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
-            }
-            elsif ( $choice eq "max_len_f_name" ) {
-                my $prompt = 'Maximum filename length';
-                my $digits = 3;
-                _opt_number_range( $opt, $choice, $prompt, $digits );
-            }
-            elsif ( $choice eq "modify_timestamp" ) {
-                my $prompt = 'Timestamp to upload date';
-                my $list = [ qw( NO YES ) ];
-                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
-            }
             elsif ( $choice eq "list_size_idx" ) {
                 my $prompt = 'Youtube videos per channel/playlist';
                 my $list = [
-                    'All',
-                    'Latest 50',
+                    'all',
+                    'latest 50',
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
@@ -390,25 +421,43 @@ sub set_options {
                 set_sort_videolist( $opt );
             }
             elsif ( $choice eq "show_view_count" ) {
-                my $prompt = 'Show the view count in channel/playlist menus';
+                my $prompt = 'Show view count in list-menus';
                 my $list = [
-                    'If sorted by "view count"',
-                    'Always',
+                    'if sorted by "view count"',
+                    'always',
+                ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
+            elsif ( $choice eq "_fast_listmenu" ) {
+                my $sub_menu = _sub_menus( $choice );
+                my $current = { map { $_->[0] => $opt->{$_->[0]} } @$sub_menu };
+                my $prompt = 'Enable fast list-menu for';
+                _opt_choose_multi( $opt, $sub_menu, $current, $prompt );
+            }
+            elsif ( $choice eq "text_unidecode" ) {
+                my $prompt = 'Unmappable characters';
+                my $list = [
+                    'replace with *',
+                    'use Text::Unidecode',
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
             elsif ( $choice eq "max_info_width" ) {
-                my $prompt = 'Maximum Info width';
+                my $prompt = 'Max Info width';
                 my $digits = 3;
                 _opt_number_range( $opt, $choice, $prompt, $digits );
             }
-
             elsif ( $choice eq "kb_sec_len" ) {
                 my ( $min, $max ) = ( 3, 9 );
                 my $prompt = 'Digits for "k/s" (download speed)';
                 my $digits = 1;
                 _opt_number_range( $opt, $choice, $prompt, $digits );
             }
+            elsif ( $choice eq "use_netrc" ) {
+                my $prompt = 'Enable option --netrc for youtube-dl';
+                my $list = [ qw( no yes ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
             else { die $choice }
         }
     }
@@ -416,6 +465,17 @@ sub set_options {
 }
 
 
+sub _opt_choose_multi {
+    my ( $opt, $sub_menu, $current, $prompt ) = @_;
+    my $changed = choose_multi( $sub_menu, $current, { prompt => $prompt, in_place => 0 } );
+    return if ! $changed;
+    for my $key ( keys %$changed ) {
+        $opt->{$key} = $changed->{$key};
+    }
+    $opt->{change}++;
+}
+
+
 sub _opt_choose_a_directory {
     my( $opt, $choice, $prompt ) = @_;
     my $new_dir = choose_a_dir( { dir => $opt->{$choice} } );
@@ -457,10 +517,8 @@ sub _opt_number_range {
     my $current = $opt->{$section};
     $current = insert_sep( $current );
     # Choose_a_number
-    my $choice = choose_a_number( $digits, { name => $prompt, current => $current } );
-    if ( ! defined $choice ) {
-        return;
-    }
+    my $choice = choose_a_number( $digits, { name => "\"$prompt\"", current => $current } );
+    return if ! defined $choice;
     $opt->{$section} = $choice eq '--' ? undef : $choice;
     $opt->{change}++;
     return;
@@ -483,11 +541,13 @@ sub _opt_choose_from_list_idx {
             $opt->{$section} = $backup;
             return;
         }
-        if ( $choices->[$idx] eq $confirm ) {
+        elsif ( $choices->[$idx] eq $confirm ) {
             $opt->{change}++;
             return;
         }
-        $opt->{$section} = $idx - @pre;
+        else {
+            $opt->{$section} = $idx - @pre;
+        }
     }
 }
 
@@ -507,21 +567,21 @@ sub _opt_choose_from_list_value {
             $opt->{$section} = $backup;
             return;
         }
-        if ( $choice eq $confirm ) {
+        elsif ( $choice eq $confirm ) {
             $opt->{change}++;
             return;
         }
-        $choice =~ s/^-\s//;
-        $opt->{$section} = $choice;
+        else {
+            $choice =~ s/^-\s//;
+            $opt->{$section} = $choice;
+        }
     }
 }
 
 
 sub _write_config_file {
     my ( $opt, $file, @keys ) = @_;
-    if ( ! $opt->{change} ) {
-        return;
-    }
+    return if ! $opt->{change};
     my $tmp = {};
     for my $section ( sort @keys ) {
         $tmp->{$section} = $opt->{$section};
